package com.ruoyi.common.auth.ali;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.alibaba.fastjson.parser.Feature;
import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.internal.util.AlipayEncrypt;
import com.alipay.api.internal.util.AlipaySignature;
import com.alipay.api.request.AlipaySystemOauthTokenRequest;
import com.alipay.api.response.AlipaySystemOauthTokenResponse;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Map;

/**
 * 支付宝认证服务
 * @author wjh
 * 2024/11/25
 */
@Service
@Slf4j
public class AliAuthService {

    @Autowired
    private AlipayClient alipayClient;

    @Autowired
    private AliConfig aliConfig;


    public String getOpenId(String loginCode) {
        // 构造请求参数以调用接口
        AlipaySystemOauthTokenRequest request = new AlipaySystemOauthTokenRequest();
        // 设置授权码
        request.setCode(loginCode);
        // 设置授权方式
        request.setGrantType("authorization_code");

        try {
            AlipaySystemOauthTokenResponse response = alipayClient.execute(request);
            System.out.println(response.getBody());
            ServiceUtil.assertion(!response.isSuccess(), response.getMsg());
            return response.getOpenId();
        } catch (Exception e) {
            throw new ServiceException(e.getMessage());
        }
    }

    /**
     * 通过密文获取到手机号
     */
    public String getPhoneNumber(String ciphertext) {
        if (StringUtils.isBlank(ciphertext)) {
            return null;
        }

        // 解密
        String str = this.decrypt(ciphertext);

        // 转为json，获取手机号
        JSONObject json = JSON.parseObject(str);
        String code = json.getString("code");
        if ( StringUtils.isBlank(code) || !code.equals("10000")) {
            log.error("获取手机号失败：" + json.getString("msg"));
            return null;
        }
        return json.getString("mobile");
    }

    /**
     * 解密
     * @param response 前端响应（密文）
     */
    private String decrypt(String response) {

        //1. 获取验签和解密所需要的参数
        Map<String, String> openapiResult = JSON.parseObject(response,new TypeReference<Map<String, String>>() {}, Feature.OrderedField);

        String signType = "RSA2";
        String charset = "UTF-8";
        String encryptType = "AES";
        String sign = openapiResult.get("sign");
        String content = openapiResult.get("response");
        //判断是否为加密内容
        boolean isDataEncrypted = !content.startsWith("{");
        boolean signCheckPass = false;
        //2. 验签
        String signContent = content;
        // 你的小程序对应的支付宝公钥（为扩展考虑建议用appId+signType做密钥存储隔离）
        String signVeriKey = aliConfig.getAlipayPublicKey();
        // 你的小程序对应的加解密密钥（为扩展考虑建议用appId+encryptType做密钥存储隔离）
        String decryptKey = aliConfig.getAesPrivateKey();//如果是加密的报文则需要在密文的前后添加双引号
        if (isDataEncrypted) {
            signContent = "\"" + signContent + "\"";
        } try {
            signCheckPass = AlipaySignature.rsaCheck(signContent, sign, signVeriKey, charset, signType);
        } catch (AlipayApiException e) {
            // 验签异常, 日志
        } if (!signCheckPass) {
            //验签不通过（异常或者报文被篡改），终止流程（不需要做解密）
            throw new ServiceException("验签失败");
        }
        //3. 解密
        String plainData = null;
        if (isDataEncrypted) {
            try {
                plainData = AlipayEncrypt.decryptContent(content, encryptType, decryptKey, charset);
            } catch (AlipayApiException e) {
                //解密异常, 记录日志
                throw new ServiceException("解密异常:" + e.getMessage());
            }
        } else {
            plainData = content;
        }

        return plainData;
    }
}
